@@ -26,6 +26,12 @@
 # Detailed usage information at the end of the file
 #
 # TODO/IDEAS: - option to call external pre-processing codes
+#
+# version 1.3.4:
+#    - Enhancement: if --flatten option is used, and no bbl file is included in the version management, then latexdiff-vc will try to run bibtex to generate the bbl file. Implements the suggestion of PR #127 (contributed by github user cousteaulecommandant )
+#    - Efficiency boost: use -draftmode for non-interactive preliminary runs of latex (idea from #
+#    - bug fix: minor fix for --only-changed that makes detection of changed pages more reliable [ PR #297 contributed by user nelijuc ]
+#
 # version 1.3.2:
 #    - bug fix: when setting config variables with the command lines it is now possible to use quotes to includes spaces in the value, e.g. -c LATEX="pdflatex --shell-escape"
 #    - bug fix: --only-changes is not compatible with graphics markup. --graphics-markup=none is now set automatically
@@ -71,7 +77,7 @@
 use Getopt::Long ;
 use Pod::Usage qw/pod2usage/ ;
 use File::Temp qw/tempdir/ ;
-use File::Basename qw/dirname/;
+use File::Basename qw/dirname fileparse/;
 use File::Copy;
 use File::Path;
 
@@ -79,8 +85,8 @@ use strict ;
 use warnings ;
 
 my $versionstring=<<EOF ;
-This is LATEXDIFF-VC 1.3.2
-  (c) 2005-2021 F J Tilmann
+This is LATEXDIFF-VC 1.3.4
+  (c) 2005-2024 F J Tilmann
 EOF
 
 # output debug and intermediate files, set to 0 in final distribution
@@ -98,6 +104,8 @@ my ($configlatexdiff,@config,$config,$assign);
 #my $latexcmd="latex"; # latex compiler if not further identified
 my $vc="";
 my $tempdir=tempdir(CLEANUP => 1);   # generate a temp dir, which will automatically be deleted at program exit
+#printf STDERR "DEBUG: tempdir $tempdir\n";
+
 # Variables
 my ($file1,$file2,$diff,$diffbase,$rootdir,$answer,$options,$infile,$append,$dirname,$cwd);
 my (@files,@ldoptions,@tmpfiles,@ptmpfiles,@difffiles,$extracomp); # ,
@@ -419,30 +427,39 @@ while ( $infile=$file2=shift @files ) {
   print STDERR "Working on  $infile \n";
   if ( scalar(@revs) == 1 ) {
     if ( defined($flatten) ) {
-       my $olddir=$tempdir . "/latexdiff-vc-$revs[0]";
-       print STDERR "Checking out old dir into: $olddir (rev: $revs[0])\n";
-       checkout_dir($revs[0],$olddir);
-       $file1=$olddir ."/".$infile;
-     } else {
-       ($file1=$infile) =~ s/\.(tex|bbl|flt)/-oldtmp-$$.$1/ ;
-       push @tmpfiles,$file1;
-       # compare file with previous version ($revs[0]="") or specified version
-       ### system("$diffcmd$revs[0] $infile| $patchcmd -o$file1") ;
-       if (system("$diffcmd$revs[0] \"$infile\" | $patchcmd -o\"$file1\"")==0  and -z $file1 ) {
-	 # no differences detected, i.e. file is equal to current version
-	 copy($infile,$file1) || die "copy($infile,$file1) failed: $!";
-       }
-     }
+      my $olddir=$tempdir . "/latexdiff-vc-$revs[0]";
+      print STDERR "Checking out old dir into: $olddir (rev: $revs[0])\n";
+      checkout_dir($revs[0],$olddir,$infile);
+      $file1=$olddir ."/".$infile;
+
+      # generate bibliography for new file if 
+      if ( greptex('^[^%]*\\\\bibliography\\{',$infile) == 0)  {
+	my ($filebase,$filedir)=fileparse($infile,".tex");
+	if ( ! -e "$filedir$filebase.bbl" ) {
+	  system("$CFG{LATEX} -draftmode -interaction=batchmode '$infile'; $CFG{BIBTEX} '$filedir$filebase'")
+	      or die  "Something went wrong in executing:  $CFG{LATEX} -draftmode -interaction=batchmode '$infile'; $CFG{BIBTEX} '$filedir$filebase'" ;
+	}
+      }
+    } else {
+      ($file1=$infile) =~ s/\.(tex|bbl|flt)/-oldtmp-$$.$1/ ;
+      push @tmpfiles,$file1;
+      # compare file with previous version ($revs[0]="") or specified version
+      ### system("$diffcmd$revs[0] $infile| $patchcmd -o$file1") ;
+      if (system("$diffcmd$revs[0] \"$infile\" | $patchcmd -o\"$file1\"")==0  and -z $file1 ) {
+        # no differences detected, i.e. file is equal to current version
+	copy($infile,$file1) || die "copy($infile,$file1) failed: $!";
+      }
+    }
   } elsif ( scalar(@revs) == 2 ) {
     if ( defined($flatten) ) {
       my $olddir=$tempdir . "/latexdiff-vc-$revs[0]";
       print STDERR "Checking out old dir into: $olddir (rev: $revs[0])\n";
-      checkout_dir($revs[0],$olddir);
+      checkout_dir($revs[0],$olddir,$infile);
       $file1=$olddir ."/".$infile;
 
       my $newdir=$tempdir . "/latexdiff-vc-$revs[1]";
       print STDERR "Checking out new dir into: $newdir\n";
-      checkout_dir($revs[1],$newdir);
+      checkout_dir($revs[1],$newdir,$infile);
       $file2=$newdir ."/".$infile;
     } else {
       ($file1=$infile) =~ s/\.(tex|bbl|flt)/-oldtmp-$$.$1/ ;
@@ -468,7 +485,7 @@ while ( $infile=$file2=shift @files ) {
   } else {
     ($diff=$infile) =~ s/\.(tex|bbl|flt)$/$append.$1/ ;
   }
-   
+
   # make directories if needed
   $dirname=dirname($diff) ;
   system("mkdir -p $dirname") unless ( -e $dirname );
@@ -488,13 +505,14 @@ while ( $infile=$file2=shift @files ) {
       die "Abort ... " ;
     }
   }
+  
   print STDERR "Running: $CFG{LATEXDIFF}  $options \"$file1\" \"$file2\" > \"$diff\"\n";
   unless ( system("$CFG{LATEXDIFF} $options \"$file1\" \"$file2\" > \"$diff\"") == 0 ) { 
     print STDERR  "Something went wrong in $CFG{LATEXDIFF}. Deleting $diff and abort\n" ; unlink $diff ; exit(5) 
   };
   print "Generated difference file $diff\n";
    
-  if ( $run and !( scalar(@revs) && greptex( qr/\\document(?:class|style)/ , $diff ) ) ) {
+  if ( $run and !( scalar(@revs) && greptex( qr/\\document(?:class|style)/ , $diff,25) ) ) {
     # save filename for later processing if postscript or pdf conversion is requested and either two-file mode was used (scalar(@revs)==0) or the diff file contains documentclass statement (ie. is a root document)
     push @difffiles, $diff ;
   }
@@ -519,19 +537,20 @@ foreach $diff ( @difffiles ) {
   if ( $run ) {
     print STDERR "PDF: $pdf Postscript: $postscript cwd $cwd\n" if $debug;
 
-    if ( system("grep -q \'^[^%]*\\\\bibliography{\' \"$diff\"") == 0 ) { 
-      system("$CFG{LATEX} --interaction=batchmode \"$diff\"; $CFG{BIBTEX} \"$diffbase\";");
+#    if ( system("grep -q \'^[^%]*\\\\bibliography{\' \"$diff\"") == 0 ) { 
+    if ( greptex('^[^%]*\\\\bibliography{',$diff ) == 0 ) { 
+      system("$CFG{LATEX} -draftmode -interaction=batchmode \"$diff\"; $CFG{BIBTEX} \"$diffbase\";");
       push @ptmpfiles, "$diffbase.bbl","$diffbase.bbl" ; 
     }
 
     # if special needs, as CHANGEBAR
     if ( $extracomp ) {
       # print "Extracomp\n";
-      system("$CFG{LATEX} --interaction=batchmode \"$diff\";");
+      system("$CFG{LATEX} -draftmode -interaction=batchmode \"$diff\";");
     }
 
     # final compilation
-    system("$CFG{LATEX} --interaction=batchmode \"$diff\";"); # needed if cross-refs
+    system("$CFG{LATEX} -draftmode -interaction=batchmode \"$diff\";"); # needed if cross-refs
     system("$CFG{LATEX} \"$diff\";"); # final, with possible error messages
 
     if ( $rundvi2 ) {
@@ -586,10 +605,10 @@ sub findchangedpages {
   my %start;
   open(AUX,$auxfile) or die ("Could open aux file $auxfile . System error: $!");
   while (<AUX>) {
-    if (m/\\zref\@newlabel\{DIFchgb(\d*)\}\{.*\\abspage\{(\d*)\}\}/ ) { 
+    if (m/\\zref\@newlabel\{DIFchgb(\d*)\}\{.*\\abspage\{(\d*)\}.*\}/ ) { 
       $start{$1}=$2; $pages{$2}=1;
     }
-    if (m/\\zref\@newlabel\{DIFchge(\d*)\}\{.*\\abspage\{(\d*)\}\}/) { 
+    if (m/\\zref\@newlabel\{DIFchge(\d*)\}\{.*\\abspage\{(\d*)\}.*\}/) { 
       if (defined($start{$1})) {
 	for ($j=$start{$1}; $j<=$2; $j++) {
 	  $pages{$j}=1;
@@ -643,12 +662,14 @@ sub compresspages {
   return @res;
 }
 
-# checkout_dir(rev,dirname)
+# checkout_dir(rev,dirname, <file>)
 # checks out revision rev and stores it in dirname
-# uses global variables: $vc, $rootdir
+# If argument <file> is provided, check if this file imports a bibliography and generate it if necessary 
+# uses global variables: $vc, $rootdir, $CFG
 sub checkout_dir {
-  my ($rev,$dirname)=@_;
+  my ($rev,$dirname,$file)=@_;
 
+  my ($filebase,$filedir);
   unless (-e $dirname) { mkpath([ $dirname ]) or die "Cannot mkdir $dirname ." ;}
   if ( $vc eq "SVN" ) {
     system("svn checkout -r $rev $rootdir $dirname")==0 or die "Something went wrong in executing:  svn checkout -r $rev $rootdir $dirname";
@@ -660,13 +681,25 @@ sub checkout_dir {
   } else {
     die "checkout_dir: only works with SVN, HG and GIT VCS system (selected: $vc)";
   }
+  # Check if the main file needs a bbl generated
+  if ( defined($file) and greptex('^[^%]*\\\\bibliography\\{',$file) == 0)  {
+      ($filebase,$filedir)=fileparse($file,".tex");
+      if ( ! -e "$filedir$filebase.bbl" ){
+ 	  printf STDERR "Running $CFG{BIBTEX} to generate $filedir$filebase.bbl.\n";
+	  system("cd '$dirname'; $CFG{LATEX} -draftmode -interaction=batchmode '$file'; $CFG{BIBTEX} '$filedir$filebase'")==0
+	      or die  "Something went wrong in executing:  cd '$dirname'; $CFG{LATEX} -draftmode -interaction=batchmode '$file'; $CFG{BIBTEX} '$filedir$filebase'" ;
+      }
+#      else { # DEBUG
+#	  printf STDERR "DEBUG: Skipping bbl file generation as $filedir$filebase.bbl already exists.\n"
+#      } #DEBUG
+  }
 }
 
-# greptex returns 1 if regex is not matched in filename
-# only the 25 first non-comment lines are scanned
+# greptex($regex,$filename, <lines>) returns 1 if regex is not matched in filename
+# only the first <lines> non-comment lines are scanned; by default everythin is scanned
 # 0 if there is a match
 sub greptex {
-  my ($regex,$filename)=@_;
+  my ($regex,$filename,$nlines)=@_;
   my ($i)=0;
   open (FH, $filename) or die("Couldn't open $filename: $!");
   while (<FH>) {
@@ -675,9 +708,9 @@ sub greptex {
       close(FH);
       return(0);
     }
-    # only scan 25 lines
+    # only scan <nlines> lines, if optional parameter proviede
     $i++;    
-    last if $i>25 ;
+    last if defined($nlines) and $i>$nlines ;
   }
   close(FH);
   return(1);
@@ -761,7 +794,7 @@ complete directory hierarchy.  Optionally, a pathname F<path> can be specified,
 =item B<--flatten,--flatten=keep-intermediate>
 
 If combined with C<--git>, C<--svn> or C<--hg> option or the corresponding modes, check out the revisions to compare in a separate temporary directory, and then pass on option C<--flatten> to latexdiff. The directory in which C<latexdiff-vc> is invoked defines the subtree which will be checked out.
-Note that if additional files are needed which are not included in the flatten procedure (package files, included graphics), they need to be accessible in the current directory. If you use bibtex, it is recommended to include the C<.bbl> file in the version management.
+Note that if additional files are needed which are not included in the flatten procedure (package files, included graphics), they need to be accessible in the current directory. If you use bibtex, it is recommended to include the C<.bbl> file in the version management. From latexdiff-vc >=1.3.4  there is an attempt to create the C<.bbl> file by running bibtex in the retrieved subdirectories; note that the bibtex is run in the temporary subdirectories so any .bib or .bst files either need to be under version management, or in the global search paths for bibtex (shell environment variables C<BIBINPUTS> and C<BSTINPUTS>). 
 
 The generic usage of this function is : C<latexdiff-vc --flatten -r rev1 [-r rev2] master.tex> where master.tex is the project file containing the highest level of includes etc.
 
@@ -798,7 +831,7 @@ Available variables for latexdiff-vc:
 
 =over 8
 
-=item C<LATEXDIFF> latexdiff command (e.g. latexdiff-fast, latexdiff-so). This command should support the option C<--interaction=batchmode>
+=item C<LATEXDIFF> latexdiff command (e.g. latexdiff-fast, latexdiff-so). This command should support the options C<-draftmode> and C<-interaction=batchmode>
 
 =item C<LATEX> latex command (e.g. pdflatex, lualatex)
 
@@ -847,7 +880,7 @@ Show version number
 
 =back
 
-All other options are passed on to C<latexdiff>.
+All other options are passed on to C<latexdiff>.  Note that it is not always easy for latexdiff to correctly assign arguments to options when short form (single letter) options are used so it is recommended to use long-form options. 
 
 =head1 SEE ALSO
 
@@ -868,7 +901,7 @@ or send them to I<tilmann -- AT -- gfz-potsdam.de>.  Include the version number
 
 =head1 AUTHOR
 
-Version 1.3.2
+Version 1.3.4a
 Copyright (C) 2005-2017 Frederik Tilmann
 
 This program is free software; you can redistribute it and/or modify
